//
//
// hello.array.44.c
//
// Charlieplex LED array hello-world
//
// Neil Gershenfeld
// 11/13/10
//
// (c) Massachusetts Institute of Technology 2010
// This work may be reproduced, modified, distributed,
// performed, and displayed for any purpose. Copyright is
// retained and must be preserved. The work is provided
// as is; no warranty is provided, and users accept all 
// liability.
//

#include <avr/io.h>
#include <util/delay.h>

#define output(directions,pin) (directions |= pin) // set port direction for output
#define input(directions,pin) (directions &= (~pin)) // set port direction for input 
#define set(port,pin) (port |= pin) // set port pin
#define clear(port,pin) (port &= (~pin)) // clear port pin
#define pin_test(pins,pin) (pins & pin) // test for port pin
#define bit_test(byte,bit) (byte & (1 << bit)) // test for bit set

#define led_delay() _delay_ms(1) // LED delay

#define led_port PORTA
#define led_direction DDRA

#define A (1 << PA1) // row 1
#define B (1 << PA2) // row 2
#define C (1 << PA3) // row 3


void flash(uint8_t from, uint8_t to, uint8_t delay) {
   //
   // source from, sink to, flash
   //
   static uint8_t i;
   set(led_port,from);
   clear(led_port,to);
   output(led_direction,from);
   output(led_direction,to);
   for (i = 0; i < delay; ++i)
       led_delay();
   input(led_direction,from);
   input(led_direction,to);
   }

void led_cycle(uint8_t number, uint8_t delay) {
   //
   // cycle through LEDs KIT style
   //
   uint8_t i;
   for (i = 0; i < number; ++i) {
      flash(A,B,delay); // led 1
      flash(B,A,delay); // led 2
      flash(B,C,delay); // led 3
      flash(C,B,delay); // led 4
      flash(C,A,delay); // led 5
      flash(A,C,delay); // led 6
      flash(C,A,delay); // led 5
      flash(C,B,delay); // led 4
      flash(B,C,delay); // led 3
      flash(B,A,delay); // led 2

      }
   }


void led_cycle3(uint8_t number, uint8_t delay) {
   //
   // Pair
   //
   uint8_t i;
   for (i = 0; i < number; ++i) {
      flash(A,B,delay); // led 1
      flash(C,A,delay); // led 5
      
      flash(B,A,delay); // led 2
      flash(C,B,delay); // led 4
     
      flash(B,C,delay); // led 3
      flash(A,C,delay); // led 5
      
      flash(B,A,delay); // led 2
      flash(C,B,delay); // led 4

      }
   }


   ISR(PCINT0_vect)
{
 if ((PINA & (1<<PINA7)) == 0)//check if a bit is set ('&'' means check, ==0 if it is set to 0)
 {
  led_cycle(1,100); //turn on LED
 } else {
  led_cycle3(1,100); //Turn of LED
 }
}


int main(void) {
   //
   // set clock divider to /1
   //
   CLKPR = (1 << CLKPCE);
   CLKPR = (0 << CLKPS3) | (0 << CLKPS2) | (0 << CLKPS1) | (0 << CLKPS0);

   //
   //
   DDRB |= (1<<DDB2); // set as output. set bit in register. LED
   DDRA &= ~(1<<DDA4); // set as input. clear bit in register. BUTTON

   PORTB &= ~(1<<PB2); // turn off LED
   PORTA |= (1<<PA4); // enable pull up resistor Button


   GIMSK |= (1<<PCIE0); //Enable pin change interrupt.
   PCMSK0 |= (1<<PCINT7); //pin mask
   sei(); // enable global interrupt

   while (1) {
     
      }
  
   }
